(function(){
    domUtil = UE.dom.domUtils;
    utils = UE.utils;

    function makeGetter(editor){
        return function(name){
            return $('#' + editor.container.id + '_' + name);
        };
    }

    UE.plugins['editor-extend'] = function(){
        var me = this;
        var getDom = makeGetter(this);
        me.commands['set-title-data'] = {
            execCommand : function( cmdName , val) {
                getDom('extendTitle')[0].value = val;
                me.fireEvent('titleChange', val, me);
            }
        };

        //附加标题输入框
        me.addListener('ready', function(){
            var toolbar = getDom('toolbarbox');
            if(this.options.showTitle == false){
                return;
            }
            // toolbar.append('<div class="editor-extend-title"><input type="text" placeholder="请在这里输入标题" value="" id="'+this.container.id + '_extendTitle'+'"/></div>');
            $('<div class="editor-extend-title"><input type="text" placeholder="请在这里输入标题" value="" id="'+this.container.id + '_extendTitle'+'"/></div>').insertAfter(toolbar);
            getDom('extendTitle').bind('keyup', function(e){
                me.fireEvent('titleChange', e.target.value, me);
            });
        });

        //切换样式
        me.commands['set-style-open'] = {
            execCommand : function( cmdName , val) {
                var el = $(this.container).find('.tool-show-style');
                el.find('.edui-label').text('关闭样式');
                el.children().attr('show', 'true');
            }
        };

        me.commands['set-style-close'] = {
            execCommand : function( cmdName , val) {
                var el = $(this.container).find('.tool-show-style');
                el.find('.edui-label').text('显示样式');
                el.children().removeAttr('show');
            }
        };
    };

    function clearNodeStyle(root){
        if(root.type == 'element'){
            if(root.children && root.children.length){
                utils.each(root.children,function(item){
                    clearNodeStyle(item);
                });
            }
            root.setAttr('style', '');
        }
    }

    function clearTextNbsp(root){
        if(root.type == 'element'){
            if(root.children && root.children.length){
                utils.each(root.children,function(item){
                    clearTextNbsp(item);
                });
            }
        }else if(root.type=='text'){
            root.data = root.data.replace(/\&[a-zA-Z]+;/g,'');
        }

    }

    function clearNodeTagName(root,tagName) {
        if(root.type === 'element'){
            if(root.children && root.children.length){
                utils.each(root.children,function (item) {
                    clearNodeTagName(item,tagName);
                });
            }
            if(root.tagName === tagName){
                toTextNode(root);
            }
        }
    }

    function clearNodeWithQQVideo(root) {
        if(root.type == 'element'){
            if(root.children && root.children.length){
                utils.each(root.children,function(item){
                    clearNodeWithQQVideo(item);
                });
            }
            if(root.tagName === 'iframe' && root.attrs.src.indexOf('v.qq.com/iframe')){
                toTextNode(root);
            }
        }
    }

    function clearNodeIframeBackgroundImg(root) {
        if(root.type == 'element'){
            if(root.children && root.children.length){
                utils.each(root.children,function(item){
                    clearNodeIframeBackgroundImg(item);
                });
            }
            if(root.tagName === 'iframe' && root.attrs.class === 'video_iframe'){
                root.attrs.style = '';
            }
        }
    }


    function toTextNode(node, text){
        node.data = text || '';
        node.children = undefined;
        node.type = 'text';
        node.tagName = undefined;
        node.attrs = {};
        return node;
    }

    var InputFilterMap = {
        'remove-style': function(root){
            utils.each(root.children,function(item){
                clearNodeStyle(item);
            });
        },
        'weibo-link': function(root){
            var aList = root.getNodesByTagName('a');
            utils.each(aList,function(item){
                var href = item.getAttr('href');
                if(href && href.indexOf('weibo.com/') > -1){

                }else{
                    console.log('not weibo link', item);
                    var text = item.innerText();
                    toTextNode(item, text);
                }
            });
        },
        'remove-nbsp':function(root) {
            utils.each(root.children,function(item){
                clearTextNbsp(item);
            });
        },
        'remove-iframe':function(root) {
            utils.each(root.children,function(item){
                clearNodeTagName(item,'iframe');
            });
        },
        'remove-qqVideo':function(root) {
            utils.each(root.children,function(item){
                clearNodeWithQQVideo(item);
            });
        },
        'remove-video':function(root) {
            utils.each(root.children,function(item){
                clearNodeTagName(item,'video');
            });
        },
        'remove-audio':function(root) {
            utils.each(root.children,function(item){
                clearNodeTagName(item,'audio');
                clearNodeTagName(item,'qqmusic');
            });
        },
        'change-qqmusic':function(root) {
            var list = root.getNodesByTagName('qqmusic');
            utils.each(list,function(item){
                var dataMusic = JSON.parse(decodeURIComponent(item.attrs['data-music']));
                console.log(dataMusic);
                console.log(item);
                item.parentNode.attrs.class = "empty"
                item.tagName = 'iframe';
                item.attrs={
                    'data-music':item.attrs['data-music'],
                    class:'music',
                    type:'music',
                    id:dataMusic.id.toString(),
                    f:'no',
                    frameborder:'0',
                    src:'https://mats.gtimg.com/om/om_2.0/libs/ueditor/iframe/music20160714.html?musicid='+dataMusic.id+'&musicname='+dataMusic.songname+'&musicinfo='+encodeURIComponent(dataMusic.singername),
                    allowfullscreen:'false',
                    style:'width:90%;height: 38px;'
                }
            });
        },
        'remove-iframe-background-img':function(root) {
            utils.each(root.children,function(item){
                clearNodeIframeBackgroundImg(item);
            });
        },
        'set-mpvoice-attr': function(root){
            var l = root.getNodesByTagName('mpvoice');
            console.log(l);
            utils.each(l,function(item){
                var _src = item.getAttr('src');
                // console.log(_src);
                // if(_src.indexOf('http://mp.weixin') < 0){
                //     var bg = 'http://mp.weixin.qq.com' + _src;
                //     item.setAttr('src', bg);
                // }
                item.setAttr('data-name', decodeURIComponent(item.getAttr('name')));
                var time = parseInt(item.getAttr('play_length'), 10);
                item.setAttr('data-length', Math.floor(time/60000) + ':' + ( (time%60000)/1000) );
            });
        }
    }

    UE.plugin.register('content-filter', function (){
        return {
            bindEvents:{
                'ready':function(){

                }
            },

           outputRule: function(root){

           },

           inputRule:function(root){
               if(this.options.inputFilter){
                   this.options.inputFilter.forEach(function(item){
                       console.log(item);
                       if(InputFilterMap[item]){
                           InputFilterMap[item](root);
                       }
                   });
               }
           },

           commands:{

           }
        }
    });

    // 图片地址映射
    UE.plugin.register('imgsr-map', function (){
        return {
           outputRule: function(root){

           },
           inputRule:function(root){
               var imgs = root.getNodesByTagName('img');
               utils.each(imgs,function(item){
                   var src = item.getAttr('src');
                   if(src.indexOf('mmbiz') > 0){
                       var _src = src.replace(/^https?:\/\/mmbiz\.(?:qlogo|qpic)\.cn/, 'http://mmbiz-qpic-cn.weituibao.com');
                       item.setAttr('src', _src);
                   }
               });
           },
           commands:{

           }
        }
    });

})();
