const co = require('co');
const platform = require('../config/platform');

function cookieToMap(list) {
    const _res = {};
    list.forEach(function (item) {
        if (item.name && item.value) {
            _res[item.name] = item.value;
        }
    });
    return _res;
}

function splitCookies(str) {
    const res = [];
    str.split(';').forEach(item => {
        const _str = item.split('=');
        const o = {
            name: _str[0].trim(),
            value: _str.splice(1, _str.length - 1).join('=')
        };
        res.push(o);
    });
    return res;
}

module.exports = {
    initState: co.wrap(function*(ctx, next) {
        const { app } = ctx;
        const response = yield app.AccountService.getAccountList('all');
        const list = [];
        const detail = {};
        const detailList = [];
        const platformCount = {}; // 已有账号的平台列表

        for (let i = 0; i < response.rows.length; i++) {
            const row = response.rows[i];
            const doc = row.doc;
            doc.data._id = doc._id;
            doc.data._rev = doc._rev;
            detail[row.id] = doc;
            detailList.push(doc);
            if (platformCount[doc.type]) {
                platformCount[doc.type]++;
            } else {
                platformCount[doc.type] = 1;
            }
        }
        // a-b < 0, a在前
        detailList.sort(function (a, b) {
            return -((a.data.order || 0) - (b.data.order || 0));
        });
        detailList.map(function (item) {
            return list.push(item._id);
        });
        ctx.body = {
            list,
            detail,
            detailList,
            platformCount
        };
        yield next();
    }),
    parseCookie: co.wrap(function*(ctx, next) {
        // debugger;
        const userInfo = ctx.message.userInfo;
        const ck = userInfo.cookies;
        const platfromInfo = platform[userInfo.type];
        let cookies = [];
        Object.keys(ck).forEach(key => {
            if (key.indexOf(platfromInfo.cookieDomain) > -1) {
                cookies = cookies.concat(splitCookies(ck[key]));
            }
        });
        // userInfo.cookies = cookies;
        // debugger;
        userInfo.cookiesMap = cookieToMap(cookies);
        yield next();
    }),
    parseWeixinInfo: co.wrap(function*(ctx, next) {
        // debugger;
        const userInfo = ctx.message.userInfo;
        if (userInfo.type != 'weixin') {
            yield next();
            return;
        }
        const ckMap = userInfo.cookiesMap;
        userInfo.avatar = `http://open.weixin.qq.com/qr/code/?username=${ckMap.ticket_id}`;
        userInfo.uid = ckMap.slave_user;
        userInfo.unid = ckMap.data_bizuin;
        console.log('userInfo', userInfo);
        const res = yield ctx.app.PlatformApi.WeiXin.getUserInfo(userInfo.token, userInfo.cookiesMap);
        console.log('wexin res', res);
        const resUser = res.user_info;
        const nav = res.base_resp.nav.nav_items[0].nav_item;
        let hasMsgcopyright = false;
        nav.forEach(item => {
            if (item.id == 10042) {
                hasMsgcopyright = true;
            }
        });
        userInfo.hasMsgcopyright = hasMsgcopyright;
        userInfo.username = resUser.nick_name;
        userInfo.ticket = res.base_resp.media_ticket;
        userInfo.ticket_id = resUser.user_name;
        yield next();
    }),

    parseQQInfo: co.wrap(function* (ctx, next) {
        const userInfo = ctx.message.userInfo;
        if (userInfo.type != 'qq') {
            yield next();
            return;
        }
        const ckMap = userInfo.cookiesMap;
        if (ckMap.ptui_loginuin) {
            userInfo.uid = ckMap.ptui_loginuin;
        } else if (ckMap.uin) {
            userInfo.uid = ckMap.uin.match(/[1-9]\d*/)[0];
        }
        console.log('set qq uid', userInfo);
        yield next();
    }),

    parseJianShuInfo: co.wrap(function* (ctx, next) {
        const userInfo = ctx.message.userInfo;
        if (userInfo.type != 'jianshu') {
            yield next();
            return;
        }
        const info = yield ctx.app.PlatformApi.JianShu.getStatistics(userInfo.cookiesMap, userInfo.uid);
        userInfo.username = info.account_name;
        userInfo.avatar = info.avatar;
        console.log('set yidian uid', userInfo);
        yield next();
    }),

    parseYiDianInfo: co.wrap(function* (ctx, next) {
        const userInfo = ctx.message.userInfo;
        if (userInfo.type != 'yidianzixun') {
            yield next();
            return;
        }
        const info = yield ctx.app.PlatformApi.YiDianZiXun.getUserInfo(userInfo);
        userInfo.uid = info.uid;
        console.log('set jianshu uid', userInfo);
        yield next();
    }),

    save: co.wrap(function* (ctx, next) {
        // debugger;
        ctx.log('添加账号');
        const app = ctx.app;
        const message = ctx.message;
        const userInfo = message.userInfo;
        const ckMap = userInfo.cookiesMap;
        const platfromInfo = platform[userInfo.type];
        if (!userInfo.uid && platfromInfo.cookieUidKey) {
            userInfo.uid = ckMap[platfromInfo.cookieUidKey];
        }
        if (!userInfo.uid) {
            throw new Error('缺少uid');
        }
        const id = `${platfromInfo.key}_${userInfo.uid}`;
        ctx.log('userInfo完成');
        let exist;
        let body;
        try {
            exist = yield app.AccountService.getAccount(id);
            exist = exist.data;
        } catch (e) {
            console.log(e);
            // 查询获取不到时，会抛出异常
        }
        if (exist) {
            ctx.log('已存在');
            userInfo.order = exist.order;
            // 登录用的账号和密码如果当前获取的信息里没有的话不能覆盖原来的
            userInfo.account = userInfo.account || exist.account || '';
            // 密码加密保存
            userInfo.password = userInfo.password
                ? app.VUtil.encodeForCloud(userInfo.password)
                : (exist.password || '');

            userInfo.loginTime = new Date();
            body = yield app.AccountService.updateAccount(id, userInfo);
            body.action = 'update';
        } else {
            ctx.log('不存在');
            userInfo.order = Date.now();
            userInfo.password = userInfo.password ? app.VUtil.encodeForCloud(userInfo.password) : '';
            userInfo.loginTime = new Date();
            body = yield app.AccountService.addAccount(userInfo.type, userInfo.uid, userInfo);
            body.action = 'add';
        }
        ctx.log('保存成功');
        ctx.body = body;
        yield next();
    })
};
