const co = require('co');
const util = require('../modules/util');
const platformConfig = require('../config/platform');
const cheerio = require('cheerio');
const path = require('path');

module.exports = {
    // 预处理，读取账号，文章信等息，基本判断
    syncPrepare: co.wrap(function* (ctx, next) {
        console.log('::sync prepare.');
        console.warn('type::::', ctx);
        const app = ctx.app;
        const message = ctx.message;
        ctx.log('同步文章到单个账号');
        try {
            const recordRes = yield app.SyncHistoryService.getHistory(ctx.message.recordId);
            ctx.record = recordRes.data;

            const accountRes = yield app.AccountService.getAccount(ctx.message.accountId);
            ctx.account = accountRes.data;

            ctx.platform = ctx.account.type;
            ctx.platApi = app.PlatformApi[platformConfig[ctx.platform].apiName];

            if (!ctx.account) {
                throw new Error('账号没有登录');
            }

            const articleIds = ctx.record.articles.map(item => item._id);
            const articlesRes = yield app.ArticleService.getArticles(articleIds);
            ctx.articlesData = articlesRes.map((item) => {
                const _item = item.data[ctx.platform];
                _item.content = util.Html.simpleFormat(_item.content);
                return _item;
            });
            yield next();
        } catch (e) {
            yield app.SyncHistoryService.setOneAccountError(message.recordId, message.accountId, e);
            throw e;
        }
    }),

    reflashWeiXinTicket: co.wrap(function* (ctx, next) {
        const { app } = ctx;
        if (ctx.account.type != 'weixin') {
            yield next();
            return;
        }
        try {
            console.log(ctx.account);
            const info = yield app.PlatformApi.WeiXin.getUserInfo(ctx.account.token, ctx.account.cookiesMap);
            console.log(info);
            const ticket = info.base_resp.media_ticket;
            ctx.account.ticket = ticket;
            yield app.AccountService.updateAccount(ctx.account._id, ctx.account);
            console.log('weixin更新ticket');
        } catch (e) {
            console.log('weixin更新ticket失败', e);
        }

        yield next();
    }),

    // 下载所有第三方图片，并且替换成本地路径
    syncDownloadImages: co.wrap(function* (ctx, next) {
        console.log('::syn download remote images');
        ctx.log('下载第三方图片到本地');
        const app = ctx.app;
        const message = ctx.message;
        const articlesData = ctx.articlesData;
        try {
            // 下载远程图片地址
            const remoteImages = util.Article.getArticlesRemoteImages(articlesData);
            const localImagesData = yield app.LocalImageService.getLocalUrl(remoteImages);
            if (localImagesData.remoteList.length) {
                const downloadList = yield app.WxbApi.downloadImages(localImagesData.remoteList, app.materialDir);
                localImagesData.downloadList = downloadList;
            }
            if (localImagesData.downloadList) {
                // 将下载的图片信息保存到数据库
                yield app.LocalImageService.addImageList(localImagesData.downloadList);
                localImagesData.localList = localImagesData.localList.concat(localImagesData.downloadList || []);
            }
            // 将远程图片地址替换成本地图片地址
            const _replaceMap = {};
            localImagesData.localList.forEach(function (img) {
                _replaceMap[img.url] = `file://${img.file}`;
            });
            articlesData.forEach(function (articleData) {
                // console.log('before replace:', articleData.content);
                articleData.content = util.Html.replaceHtmlImagesUrl(articleData.content, _replaceMap);
                // console.log('after replace:', articleData.content);
            });
            ctx.log('第三方图片下载完成');
            yield next();
            return;
        } catch (e) {
            yield app.SyncHistoryService.setOneAccountError(message.recordId, message.accountId, e);
            throw e;
        }
    }),

    // 将所有本地图片上传，并文章中替换成对应平台的路径
    syncUploadImages: co.wrap(function* (ctx, next) {
        ctx.log('上传本地图片到平台');
        const app = ctx.app;
        const message = ctx.message;
        const articlesData = ctx.articlesData;
        const localImages = util.Article.getArticlesLocalImages(articlesData, ctx.platform);
        const platImages = yield app.LocalImageService.getPlatformUrl(localImages, ctx.platform);
        if (platImages.localList.length) {
            try {
                platImages.uploadImages = yield ctx.platApi.uploadImages(ctx.account, platImages.localList);
                console.log('done upload::', platImages.uploadImages);
                yield app.LocalImageService.addImageList(platImages.uploadImages); // 上传之后写入本地图片数据库
            } catch (e) {
                let _e = e;
                console.log('upload image error::');
                if (e && e.message == 'system error') {
                    console.log('过滤掉此处错误');
                    return;
                }
                yield app.SyncHistoryService.setOneAccountError(message.recordId, message.accountId, _e);
                throw _e;
            }
        }
        // 替换路径
        const replaceMap = {};
        platImages.remoteList = platImages.remoteList.concat(platImages.uploadImages || []);
        platImages.remoteList.forEach(function (img) {
            replaceMap[`file://${img.file}`] = img[ctx.platform];
        });
        console.log('replace map::', replaceMap);
        // 替换文章里的所有content数据
        articlesData.forEach(function (articleData) {
            // console.log('before replace:', articleData.content);
            articleData.content = util.Html.replaceHtmlImagesUrl(articleData.content, replaceMap);
            // console.log('after replace:', articleData.content);
        });
        ctx.platImages = platImages;
        ctx.replaceMap = replaceMap;
        ctx.log('正文图片上传完成');

        // 头条，omqq，一点资讯的处理方式比较特殊，后续单独处理
        if (['toutiao', 'omqq', 'yidianzixun'].indexOf(ctx.platform) > -1) {
            yield next();
            return;
        }

        // 处理封面
        ctx.log('替换封面');
        for (let i = 0; i < articlesData.length; i++) {
            const _article = articlesData[i];
            util.Article.replacePlatformCoverUrl(_article,
                replaceMap, platImages.remoteList);
        }

        yield next();
    }),

    syncTouTiaoCover: co.wrap(function* (ctx, next) {
        if (ctx.platform !== 'toutiao') {
            yield next();
            return;
        }
        const { articlesData, replaceMap, platApi, app, message } = ctx;
        ctx.log('头条封面处理');
        try {
            for (let i = 0; i < articlesData.length; i++) {
                const _article = articlesData[i];
                if (_article.covers) {
                    const uploadCover = yield platApi.uploadCovers(_article.covers, replaceMap, ctx.account);
                    articlesData[i].covers = undefined;
                    articlesData[i].pgc_feed_covers = uploadCover;
                } else if (!_article.covers) {
                    articlesData[i].pgc_feed_covers = [];
                }
            }
        } catch (e) {
            yield app.SyncHistoryService.setOneAccountError(message.recordId, message.accountId, e);
            throw e;
        }
        yield next();
    }),

    syncYiDianCover: co.wrap(function* (ctx, next) {
        if (ctx.platform !== 'yidianzixun') {
            yield next();
            return;
        }
        const { articlesData, replaceMap, platApi, app, message } = ctx;
        ctx.log('一点资讯封面处理');
        try {
            for (let i = 0; i < articlesData.length; i++) {
                const _article = articlesData[i];
                if (_article.covers) {
                    const uploadCover = yield platApi.uploadCovers(_article.covers, replaceMap, ctx.account);
                    // articlesData[i].covers = undefined;
                    articlesData[i].covers = uploadCover;
                } else if (!_article.covers) {
                    articlesData[i].covers = [];
                }
            }
        } catch (e) {
            yield app.SyncHistoryService.setOneAccountError(message.recordId, message.accountId, e);
            throw e;
        }
        yield next();
    }),

    syncOMQQCover: co.wrap(function* (ctx, next) {
        if (ctx.platform !== 'omqq') {
            yield next();
            return;
        }
        const { articlesData, platApi, app, message, replaceMap } = ctx;
        ctx.log('omqq封面处理');
        try {
            for (let i = 0; i < articlesData.length; i++) {
                const _article = articlesData[i];
                // debugger;
                const _articleImgs = util.Html.getRemoteImagesUrl(_article.content);
                const cover = [];
                if (!_articleImgs.length) {
                    throw new Error('文章中没有图片,无法设置封面');
                }
                if (_article.covers && _article.covers.length) {
                    for (let coverI = 0; coverI < _article.covers.length; coverI++) {
                        const coverItem = _article.covers[coverI];
                        const upRes = yield platApi.cropupload(ctx.account, {
                            url: replaceMap[coverItem.sourceurl],
                            x: coverItem.x,
                            y: coverItem.y,
                            width: coverItem.width,
                            height: coverItem.height,
                            opCode: 155
                        });
                        upRes.index = 0;
                        cover.push(upRes);
                    }
                    if (cover.length == 3) {
                        articlesData[i].cover_type = 3;
                    }
                } else if (!_article.covers) {
                    const cropRes = yield platApi.exactUpload(ctx.account, {
                        url: _articleImgs[0]
                    });
                    cover.push(cropRes);
                    articlesData[i].cover_type = -1;
                }
                articlesData[i].imgurl_ext = JSON.stringify(cover);
            }
        } catch (e) {
            yield app.SyncHistoryService.setOneAccountError(message.recordId, message.accountId, e);
            throw e;
        }
        yield next();
    }),

    // 处理微信的语音内容
    syncWeiXinVoice: co.wrap(function* (ctx, next) {
        if (ctx.platform !== 'weixin') {
            yield next();
            return;
        }
        ctx.log('上传语音');
        const app = ctx.app;
        const articlesData = ctx.articlesData;
        const message = ctx.message;
        try {
            for (let i = 0; i < articlesData.length; i++) {
                const article = articlesData[i];
                const $ = cheerio.load(article.content, { decodeEntities: false });
                const mpvoice = $('mpvoice');
                if (!mpvoice.length) continue;
                for (let voiceI = 0; voiceI < mpvoice.length; voiceI++) {
                    const dom = $(mpvoice[voiceI]);
                    const mediaid = dom.attr('voice_encode_fileid');
                    let name = dom.attr('data-name');

                    let localFile;
                    let localFileRecord;
                    let localFileRecordId;
                    let localExist;
                    let url;
                    if (!mediaid && dom.attr('src').indexOf('file:') == 0) {
                        ctx.log('本地语音');
                        // 本地选取的音频
                        localFile = dom.attr('src');
                        localExist = yield app.LocalImageService.getImagebyPath(localFile);
                    } else {
                        ctx.log('下载语音');
                        url = `http://res.wx.qq.com/voice/getvoice?mediaid=${mediaid}`;
                        // 获取本地的文件
                        localExist = yield app.LocalImageService.getImagebyUrl(url);
                        if (!localExist.rows.length) {
                            localFile = yield app.WxbApi.downloadFile(url,
                                path.join(app.materialDir, `${Date.now()}.mp3`));
                            console.log('download mp3::', localFile);
                        }
                    }

                    if (!localExist.rows.length) {
                        localFileRecord = yield app.LocalImageService.addImage({
                            file: util.Base.addFileProtocol(localFile) || '',
                            url: url || ''
                        });
                        localFileRecordId = localFileRecord.id;
                    } else {
                        localFileRecordId = localExist.rows[0].key._id;
                        localFileRecord = localExist.rows[0].key.data;
                        localFile = localFile || localFileRecord.file;
                    }

                    // 上传到微信
                    let voiceRes;
                    localFile = util.Base.removeFileProtocol(localFile);
                    if (localFileRecord[ctx.account._id]) {
                        console.log('has local voice::');
                        voiceRes = localFileRecord[ctx.account._id];
                    } else {
                        console.log('no upload,', localFile);
                        voiceRes = yield ctx.platApi.createVoice(ctx.account, {
                            file: localFile,
                            title: name
                        });
                        // voiceRes.play_length = yield util.Media.meta(localFile);
                        // voiceRes.play_length.duration * 1000;
                        voiceRes.play_length = dom.attr('play_length');
                        const _attr = {};
                        _attr[ctx.account._id] = voiceRes;
                        yield app.LocalImageService.updateItem(localFileRecordId, _attr);
                    }
                    ctx.log('语音上传成功');
                    console.log('voice::', voiceRes);
                    name = encodeURIComponent(name);
                    const len = voiceRes.play_length;
                    const voiceId = voiceRes.encode_file_id;
                    // const time = `${Math.floor(len / 60000)}:${(len % 60000) / 1000}`;
                    const time = dom.attr('data-length');
                    const frame = $('<mpvoice frameborder="0" class="res_iframe js_editor_audio audio_iframe" '
                        + `src="/cgi-bin/readtemplate?t=tmpl/audio_tmpl&amp;name=${name}&amp;play_length=${time}" `
                        + `name="${name}" play_length="${len}" voice_encode_fileid="${voiceId}"></mpvoice>`);
                    dom.replaceWith(frame);
                }
                article.content = $.html();
            }
        } catch (e) {
            yield app.SyncHistoryService.setOneAccountError(message.recordId, message.accountId, e);
            throw e;
        }
        yield next();
    }),

    // 同步文章内容到对应账号平台
    syncArticle: co.wrap(function* (ctx, next) {
        ctx.log('发送文章');
        const app = ctx.app;
        const articlesData = ctx.articlesData;
        const message = ctx.message;
        const type = ctx.record.type;
        let remoteArticle;
        try {
            if (type == 1) {
                // 发送单图文
                ctx.log('单图文');
                remoteArticle = yield ctx.platApi.postArticle(ctx.account, articlesData[0]);
            } else if (type == 2) {
                // 发送多图文
                ctx.log('多图文');
                const _articles = articlesData.map(item => util.Article.removeUncheckedAttr(item));
                remoteArticle = yield ctx.platApi.postArticles(ctx.account, _articles);
            }
        } catch (e) {
            yield app.SyncHistoryService.setOneAccountError(message.recordId, message.accountId, e);
            throw e;
        }

        console.log('remoteArticle::', remoteArticle);
        // 同步成功之后标记账户的状态
        ctx.account.status = 3;
        ctx.account.message = '同步成功';
        yield app.SyncHistoryService.updateAccountStatus(ctx.message.recordId, [ctx.account]);
        yield next();
    })
};
