'use strict';

const isGeneratorFunction = require('is-generator-function');
const convert = require('koa-convert');
const Emitter = require('events');
const compose = require('koa-compose');
const context = require('./context');

module.exports = class Application extends Emitter {

    constructor() {
        super();
        this.middleware = [];
        this.context = Object.create(context);
    }

    use(fn) {
        if (typeof fn !== 'function') throw new TypeError('middleware must be a function!');
        if (isGeneratorFunction(fn)) {
            fn = convert(fn);
        }
        this.middleware.push(fn);
        return this;
    }

    sendMessage(message, callback) {
        const fn = compose(this.middleware);
        const ctx = this.createContext(message);
        return fn(ctx).then(() => {
            if (callback) callback(ctx);
            return ctx;
        }).catch(ctx.onerror);
    }

    createContext(message) {
        const ctx = Object.create(this.context);
        ctx.message = message;
        ctx.app = this;
        ctx.name = message.name;
        ctx.body = Object.assign({}, message.body); // 约定为需要处理的数据内容
        ctx.onerror = ctx.onerror.bind(ctx);
        return ctx;
    }
};
