const html = require('./html');
const base = require('./base');
const _ = require('lodash');
const {
    getImagesUrl,
    replaceHtmlImagesUrl,
    getLocalImagesUrl,
    getRemoteImagesUrl,
    simpleFormat
} = html;
const {
    removeFileProtocol
} = base;

// 获取文章中的所有图片url
function getArticleImages(article) {
    let l = [];
    Object.keys(article).forEach(function (key) {
        const item = article[key];
        if (item && item.content) {
            l = l.concat(getImagesUrl(item.content));
        }
    });
    return _.union(l); // 去重
}

// 将文章中的url地址
function replaceImagesUrl(article, images) {
    const values = Object.values(article);
    values.forEach(function (val) {
        if (val.content) {
            val.content = replaceHtmlImagesUrl(val.content, images);
        }
    });
    return article;
}


function getArticleCoverUrls(article) {
    const l = [];
    Object.values(article).forEach(val => {
        if (!val) {
            return;
        }
        const cover = val.cover;
        const covers = val.covers;
        if (cover && _.isString(cover)) {
            l.push(cover);
        } else if (cover && _.isObject(cover)) {
            l.push(removeFileProtocol(cover.file));
        } else if (covers) {
            covers.forEach(item => {
                l.push(removeFileProtocol(item.file));
            });
        }
    });
    return l;
}

function getPlatformCover(platformArticle) {
    const cover = platformArticle.cover;
    const covers = platformArticle.covers;
    if (cover && _.isString(cover)) {
        return [removeFileProtocol(cover)];
    } else if (cover && _.isObject(cover)) {
        return [removeFileProtocol(cover.file)];
    } else if (covers) {
        return [];
    }
    return null;
}

function replaceArticleCoverUrls(article, urlMap) {
    Object.keys(article).forEach(key => {
        const val = article[key];
        if (!val) return;
        const cover = val.cover;
        let covers = val.covers;
        if (cover && _.isString(cover) && urlMap[cover]) {
            article[key].cover = urlMap[cover];
        } else if (cover && _.isObject(cover)) {
            article[key].cover.file = urlMap[cover.file];
        } else if (covers) {
            covers = covers.map(item => {
                item.file = urlMap[item.file];
                return item;
            });
            article[key].covers = covers;
        }
    });
    return article;
}

function converToCloudUpload(article) {
    Object.keys(article).forEach(key => {
        const val = article[key];
        const cover = val.cover;
        if (cover && _.isObject(cover)) {
            article[key].cover = cover.file;
        }
    });
    return article;
}

function converToLocal(article) {
    Object.keys(article).forEach(key => {
        const val = article[key];
        const cover = val.cover;
        if (cover && _.isString(cover)) {
            article[key].cover = {
                file: cover
            };
        }
    });
    return article;
}

function replacePlatformCoverUrl(article, replaceMap, remoteList) {
    const cover = article.cover;
    if (cover && _.isObject(cover)) {
        cover.url = replaceMap[cover.file]; // 上传文章时要使用的实际url
        const remote = remoteList.filter(item => item.file == removeFileProtocol(cover.file));
        Object.assign(cover, remote[0]);
        article.cover = cover;
        return article;
    } else if (cover && _.isString(cover)) {
        article.cover = replaceMap[cover] || article.cover;
        return article;
    }
    return article;
}

// 获取一组文章中的所有图片数据，包括content和封面图片,去重
function getArticlesImages(articles) {
    let conImgs = [];
    let coverImgs = [];
    articles.forEach(article => {
        conImgs = conImgs.concat(getImagesUrl(article.content));
        const _cover = getPlatformCover(article);
        if (_cover && _cover.length) {
            coverImgs = coverImgs.concat(_cover);
        }
    });
    const images = _.uniq(conImgs.concat(coverImgs)); // 所有文章里的图片, 字符串数组，去重;
    return images;
}


// 获取本地图片路径
function getArticlesLocalImages(articles) {
    let conImgs = [];
    let coverImgs = [];
    articles.forEach(article => {
        conImgs = conImgs.concat(getLocalImagesUrl(article.content));
        const _cover = getPlatformCover(article);
        if (_cover && _cover.length) {
            coverImgs = coverImgs.concat(_cover);
        }
    });
    coverImgs = coverImgs.filter(item => item.indexOf('http') < 0);
    const images = _.uniq(conImgs.concat(coverImgs)); // 所有文章里的图片, 字符串数组，去重;
    return images;
}

function getArticlesRemoteImages(articles) {
    let conImgs = [];
    let coverImgs = [];
    articles.forEach(article => {
        conImgs = conImgs.concat(getRemoteImagesUrl(article.content));
        const _cover = getPlatformCover(article);
        if (_cover && _cover.length) {
            coverImgs = coverImgs.concat(_cover);
        }
    });
    coverImgs = coverImgs.filter(item => item.indexOf('http') == 0);
    const images = _.uniq(conImgs.concat(coverImgs));
    return images;
}


function getPlatformKeys(article) {
    const res = [];
    Object.keys(article).forEach(item => {
        const val = article[item];
        if (val && val.title && val.content) {
            res.push(item);
        }
    });
    return res;
}

function hasPlatformContent(article, checkList) {
    const hasKeys = getPlatformKeys(article);
    let has = 0;
    checkList.forEach(item => {
        if (hasKeys.includes(item)) {
            has++;
        }
    });
    return has == checkList.length;
}

// 文章数据中有一个checkMap，如果为false的字段则去除
function removeUncheckedAttr(article) {
    if (article.checkMap) {
        const checkMap = article.checkMap;
        Object.keys(checkMap).forEach(key => {
            if (checkMap[key] == false) {
                delete article[key];
            }
        });
    }
    return article;
}

// 简单处理，去除换行
function contentSimpleFormat(article) {
    Object.keys(article).forEach(key => {
        const item = article[key];
        if (item && item.content) {
            item.content = simpleFormat(item.content);
        }
    });
    return article;
}

module.exports = {
    getArticleImages,
    getArticlesLocalImages,
    replaceImagesUrl,
    getArticleCoverUrls,
    replaceArticleCoverUrls,
    converToCloudUpload,
    converToLocal,
    getPlatformCover,
    replacePlatformCoverUrl,
    getArticlesImages,
    getPlatformKeys,
    hasPlatformContent,
    removeUncheckedAttr,
    getArticlesRemoteImages,
    contentSimpleFormat
};
